@@ -32,21 +32,42 @@ class PreprocessResponse:
         }
         response = PreprocessResponse(length=len(preprocessed_data), data=preprocessed_data)
     """
-    length: int
-    data: Any
+    length: Optional[int] = None  # Deprecated. Please use sample_ids instead
+    data: Any = None
+    sample_ids: Optional[Union[List[str], List[int]]] = None
+    state: Optional[DataStateType] = None
+    sample_id_type: Optional[Union[Type[str], Type[int]]] = None
+
+    def __post_init__(self) -> None:
+        if self.length is not None and self.sample_ids is None:
+            self.sample_ids = [i for i in range(self.length)]
+            self.sample_id_type = int
+        elif self.length is None and self.sample_ids is not None:
+            self.length = len(self.sample_ids)
+            if self.sample_id_type is None:
+                self.sample_id_type = str
+        else:
+            raise Exception("length is deprecated.")
+
+    def __hash__(self) -> int:
+        return id(self)
+
+    def __len__(self) -> int:
+        assert self.sample_ids is not None
+        return len(self.sample_ids)
 
 
-SectionCallableInterface = Callable[[int, PreprocessResponse], npt.NDArray[np.float32]]
+SectionCallableInterface = Callable[[Union[int, str], PreprocessResponse], npt.NDArray[np.float32]]
 
 MetadataSectionCallableInterface = Union[
-    Callable[[int, PreprocessResponse], int],
-    Callable[[int, PreprocessResponse], Dict[str, int]],
-    Callable[[int, PreprocessResponse], str],
-    Callable[[int, PreprocessResponse], Dict[str, str]],
-    Callable[[int, PreprocessResponse], bool],
-    Callable[[int, PreprocessResponse], Dict[str, bool]],
-    Callable[[int, PreprocessResponse], float],
-    Callable[[int, PreprocessResponse], Dict[str, float]]
+    Callable[[Union[int, str], PreprocessResponse], int],
+    Callable[[Union[int, str], PreprocessResponse], Dict[str, int]],
+    Callable[[Union[int, str], PreprocessResponse], str],
+    Callable[[Union[int, str], PreprocessResponse], Dict[str, str]],
+    Callable[[Union[int, str], PreprocessResponse], bool],
+    Callable[[Union[int, str], PreprocessResponse], Dict[str, bool]],
+    Callable[[Union[int, str], PreprocessResponse], float],
+    Callable[[Union[int, str], PreprocessResponse], Dict[str, float]]
 ]
 
 
@@ -96,31 +117,46 @@ MetricCallableReturnType = Union[Any, List[List[ConfusionMatrixElement]]]
 
 
 @dataclass
-class CustomLossHandler:
+class CustomLossHandlerData:
     name: str
-    function: CustomCallableInterface
     arg_names: List[str]
 
 
 @dataclass
-class MetricHandler:
+class CustomLossHandler:
+    custom_loss_handler_data: CustomLossHandlerData
+    function: CustomCallableInterface
+
+
+@dataclass
+class MetricHandlerData:
     name: str
-    function: Union[CustomCallableInterfaceMultiArgs, ConfusionMatrixCallableInterfaceMultiArgs]
     arg_names: List[str]
     direction: Optional[MetricDirection] = MetricDirection.Downward
 
 
+@dataclass
+class MetricHandler:
+    metric_handler_data: MetricHandlerData
+    function: Union[CustomCallableInterfaceMultiArgs, ConfusionMatrixCallableInterfaceMultiArgs]
+
+
 @dataclass
 class RawInputsForHeatmap:
     raw_input_by_vizualizer_arg_name: Dict[str, npt.NDArray[np.float32]]
 
 
 @dataclass
-class VisualizerHandler:
+class VisualizerHandlerData:
     name: str
-    function: VisualizerCallableInterface
     type: LeapDataType
     arg_names: List[str]
+
+
+@dataclass
+class VisualizerHandler:
+    visualizer_handler_data: VisualizerHandlerData
+    function: VisualizerCallableInterface
     heatmap_function: Optional[Callable[..., npt.NDArray[np.float32]]] = None
 
 
@@ -133,7 +169,7 @@ class DatasetBaseHandler:
 @dataclass
 class InputHandler(DatasetBaseHandler):
     shape: Optional[List[int]] = None
-
+    channel_dim: Optional[int] = -1
 
 @dataclass
 class GroundTruthHandler(DatasetBaseHandler):
@@ -181,5 +217,5 @@ class DatasetSample:
     inputs: Dict[str, npt.NDArray[np.float32]]
     gt: Optional[Dict[str, npt.NDArray[np.float32]]]
     metadata: Dict[str, Union[str, int, bool, float]]
-    index: int
+    index: Union[int, str]
     state: DataStateEnum
