@@ -26,6 +26,12 @@ class LeapDataType(Enum):
     ImageMask = 'ImageMask'
     TextMask = 'TextMask'
     ImageWithBBox = 'ImageWithBBox'
+    ImageWithHeatmap = 'ImageWithHeatmap'
+
+
+class MetricDirection(Enum):
+    Upward = "Upward"
+    Downward = "Downward"
 
 
 class DatasetMetadataType(Enum):
